#!/usr/bin/env python
"""
Smart RAW Archiver by Temperature is a tool to archive RAW files into several sub folders named as temperature. The temperature is read from the exif data of the RAW files.

This program need to call ExifTool by Phil Harvey which you can download here:
http://www.sno.phy.queensu.ca/~phil/exiftool/
Supported RAW formats: PEF, CR2, DNG(Pentax K30)
To run the Archiver, go into the folder with RAW files in, and run srat.py
 
Usage:
    srat.py [-p] [-h]
    -h  Show help
    -p  Read exif data and generate the report only. No file will be archived.
 
Copyright Zachary Wu
          ecloud@163.com
Licensed under GPL v3
"""

# Basic configuration can be changed here:
# The MAX temperature must be greater than MIN, The INTERVAL must be greater than 0. All the values must be integers. For example: 40, 5, 4
TEMPMAX    = 37
TEMPMIN    = 12
INTERVAL   = 3
# DELTA is the range +/- a temperature step. It may be a positive floating point number. For example: 0.7
# For light frames we usually set DELTA to the half of INTERVAL like this:
# DELTA      = INTERVAL/2
DELTA      = 1.1
# Camera brand:[1 : Pentax PEF] [2 : Canon CR2] [3 : Pentax DNG]
BRAND      = 1


import sys
import string
import os
import subprocess
import time
__version__ = 1.03


# Check the options
logonly = False
if len(sys.argv) > 1:
    if sys.argv[1] == '-p':
        logonly = True
    elif sys.argv[1] == '-h':
        print __doc__
        exit(0)
    else:
        print 'Unknown option ' + str(sys.argv[1:])
        exit(1)


# Check the besic configuration.
def checkconfig():
    if TEMPMAX <= TEMPMIN:
        return 1
    if type(INTERVAL) != type(12) or INTERVAL < 1:
        return 1
    if DELTA <= 0 or DELTA > float(INTERVAL)/2:
        return 1
    if BRAND not in [1, 2, 3]:
        return 1

checkpass = checkconfig()
if checkpass == 1:
    print 'Configuration wrong. Please double check!'
    exit(1)


# The following values should NOT be modified!
TAGDICT    = {1:['MakerNotes::SensorTemperature', 'MakerNotes::SensorTemperature2'], 2:['MakerNotes::CameraTemperature'], 3:['MakerNotes::SensorTemperature', 'MakerNotes::SensorTemperature2']}
FORMATDICT = {1:['.pef', '.PEF'], 2:['.cr2', '.CR2'], 3:['.dng', '.DNG']}
REPORT     = 'report.txt'
TEMPTAGS   = TAGDICT[BRAND]
FILEFORMAT = FORMATDICT[BRAND]



# To get the list of prefered RAW files in the current directory.
filelist = os.listdir('.')
rawfiles = []
for files in filelist:
    if os.path.getsize(files) == 0:
        continue
    if os.path.splitext(files)[1] in FILEFORMAT:
        rawfiles.append(files)

# Store all the temperature data to a map.
tempmap=dict()
print 'Reading EXIF:.',

for rawfile in rawfiles:
    
    # Compute the average temperature for each RAW file.
    # Walk through the TEMPTAGS to read all the temperature in the exif.
    sys.stdout.write('.')
    sys.stdout.flush()
    atemp = 0.0
    for tag in TEMPTAGS:
        output = subprocess.Popen('exiftool ' + '-' + tag + ' -b ' + rawfile, shell=True, stdout=subprocess.PIPE)
        atemp = atemp + float(output.stdout.read())
    atemp = round((atemp / len(TEMPTAGS)), 3)
    # Put the average temperature into the map.
    tempmap[rawfile]=atemp
print 'Done.'


# Generate report.
print 'Generate report...'
sortedtemp = sorted(tempmap.iteritems(), key=lambda tempmap:tempmap[1], reverse=True)
if os.path.exists(REPORT):
    os.remove(REPORT)
reportfile = open(REPORT, 'w')
reportfile.write('Total ' + str(len(sortedtemp)) + ' files are processed in ' + os.path.realpath('.') + '\n')
for record in sortedtemp:
    reportfile.write(record[0].ljust(24) + ':\t' + str(record[1]) + ' C\n')
reportfile.close()


# Exit if -p is passed to the programe.
if logonly:
    exit(0)


# Populate sub folders.
if not os.path.isdir('overheat'):
    os.mkdir('overheat')

if not os.path.isdir('cool'):
    os.mkdir('cool')

folderlist = range(TEMPMAX, TEMPMIN - 1, 0 - INTERVAL)
for num in folderlist:
    if not os.path.isdir(str(num)):
        os.mkdir(str(num))


# Move the RAW files to the sub folders.
print 'Archive RAW files.'
for rawfile in tempmap:
    if tempmap[rawfile] > TEMPMAX + DELTA:
        os.rename(rawfile, 'overheat/'+rawfile)
        continue
    if tempmap[rawfile] <= folderlist[-1] - DELTA:
        os.rename(rawfile, 'cool/'+rawfile)
        continue
    for num in folderlist:
        if tempmap[rawfile] <= num + DELTA and tempmap[rawfile] > num - DELTA:
            os.rename(rawfile, str(num) + '/' + rawfile)


#Purge the empty folders.
if len(os.listdir('overheat')) == 0:
    os.rmdir('overheat')
if len(os.listdir('cool')) == 0:
    os.rmdir('cool')
for num in folderlist:
    if len(os.listdir(str(num))) == 0:
        os.rmdir(str(num))

time.sleep(1)
print 'Done.'
